/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.IIOP;

import ORG.omg.CORBA.OctetSequence;
import ORG.omg.CORBA.Streamable;
import ORG.omg.CORBA.EMPTYByteArray;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * [CORBA 2.0 Specification 12-28]
 *
 * @since ACT 0.1
 */
public final class ProfileBody implements Streamable
{
    /**
     * @since ACT 0.1
     */
    public Version iiop_version_ = Version.Version_1_0;

    /**
     * @since ACT 0.1
     */
    public String host_;

    /**
     * @since ACT 0.1
     */
    public short port_;

    /**
     * @since ACT 0.1
     */
    public byte[] object_key_ = EMPTYByteArray.EMPTY; // sequence<octet>

    /**
     * @since ACT 0.1
     */
    public boolean byte_order_ = true;

    /**
     * @since ACT 0.1
     */
    private int encapsulationSize_;

    /**
     * @since ACT 0.1
     */
    public ProfileBody() {
    }

    /**
     * @param String host
     * @param port port
     * @param byte[] object_key
     *
     * @since ACT 0.1
     */
    public ProfileBody(
        String host,
        short port,
        byte[] object_key) {
	    host_ = host;
	    port_ = port;
	    object_key_ = object_key;
    }


    /**
     * @since ACT 0.1
     */
    public int getEncapsulationSize() {
        return encapsulationSize_;
    }

    /**
     * Read from the TypeReader stream. Since the ProfileBody is an encapsulation,
     * the TypeReader index and byte ordering are first stored, reset and restored
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#read
     * @see ORG.omg.CORBA.io.TypeReader
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in) 
        throws java.io.IOException {
        // back up
        boolean preByteOrder = in.getByteOrder();
        int preIndex = in.getIndex();

        // reset alignment and byte order
        in.setIndex(0);
        byte_order_ = in.readBoolean();
        in.setByteOrder(byte_order_);

        // contents
        iiop_version_ = new Version();
        iiop_version_.read(in);
        host_ = in.readString();
        port_ = in.readShort();
        int len = in.readULong();
        object_key_ = new byte[len];
        in.readOctetArray(object_key_, 0, len);

        // restore
        encapsulationSize_ = in.getIndex();
        in.setByteOrder(preByteOrder);
        in.setIndex(preIndex + encapsulationSize_);
    }

    /**
     * Write to the TypeWriter stream. Since the ProfileBody is an encapsulation,
     * the TypeReader index and byte ordering are first stored, reset and restored.
     * In the GIOPLocateReplyMessage, there is somce sample code about how to compose
     * the IOR.
     *
     * @see EDU.rutgers.ACT.GIOP.GIOPLocateReplyMessage#setLocateReplyBody
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#write
     * @see ORG.omg.CORBA.io.TypeWriter
     * @see EDU.rutgers.ACT.IOP.IOR
     *
     * @since ACT 0.1
     */
    public void write(TypeWriter out)
        throws java.io.IOException {
        // backup
        boolean preByteOrder = out.getByteOrder();
        int preIndex = out.getIndex();

        // contents
        out.setIndex(0);
        out.writeBoolean(byte_order_);
        out.setByteOrder(byte_order_);

        iiop_version_.write(out);
        out.writeString(host_);
        out.writeShort(port_);
        int len = object_key_.length;
        out.writeULong(len);
        out.writeOctetArray(object_key_, 0, len);

        // restore
        encapsulationSize_ = out.getIndex();
        out.setByteOrder(preByteOrder);
        out.setIndex(preIndex);
    }

    /**
     * Returns a string representation. 
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        StringBuffer buf = new StringBuffer("IIOP:ProfileBody").
            append("\n\tversion: ").append(iiop_version_.toString()).
            append("\n\thost: ").append(host_).
            append("\n\tport: ").append(port_).
            append("\n\tobject_key: ").append(OctetSequence.byteArray2String(object_key_));
        return String.valueOf(buf);
    }
}
