// last update: 08/16/97

package EDU.rutgers.CORBA.clntsrv.PortableServer;

import java.io.*;
import ORG.omg.CORBA.*;
import EDU.rutgers.act.presentation.*;
import EDU.rutgers.CORBA.clntsrv.PortableServer.POAPackage.*;
import EDU.rutgers.act.clntsrv.threadengine.Service;
import EDU.rutgers.CORBA.presentation.*;
import EDU.rutgers.CORBA.IOP.*;
import EDU.rutgers.CORBA.IIOP.*;

public class POA implements Service 
{
  ObjectMap objectMap_;
  int internalId_ = 0;
  String host_;
  short port_;

  final int INITIAL_CAPACITY = 1024;
  final float LOAD_FACTOR = (float)0.8;

  // constructors
  public POA(String host, short port)
    {
      host_ = host;
      port_ = port;
      objectMap_ = new ObjectMap(INITIAL_CAPACITY, LOAD_FACTOR);
    }
  
  //
  // public methods
  //

  // the dispatching method
  public void handleRequest(EDU.rutgers.act.protocol.Message m, 
			    EDU.rutgers.act.protocol.Sender sender)
    {
      EDU.rutgers.CORBA.GIOP.messages.GIOPRequestMessage request;
      if(!(m instanceof EDU.rutgers.CORBA.GIOP.messages.GIOPRequestMessage))
	{ 
	  System.err.println("Only RequestMessage is recognized by now");
	  return;
	}
      request = (EDU.rutgers.CORBA.GIOP.messages.GIOPRequestMessage)m;
      byte[] objectId = request.object_key_;
      ObjectMapInfo objectMapInfo = objectMap_.get(objectId);
      if(objectMapInfo == null)
	{
	  byte[] aux = {1};

	  Servant servant = objectMap_.get((byte[])aux).servant;
	  servant.handleRequest(m, sender);

      
	  // send a GIOPReplyMessage with OBJECT_NOT_HERE
	  System.err.println("Object *" + new String(objectId) + "* not here!");
	  return;
	}

      if(!objectMapInfo.isActive)
	{
	  // depending on the policy activate the object or reply with
	  // some exception
	  return;
	}
      
      Servant servant = objectMapInfo.servant;
      servant.handleRequest(m, sender);
    }
  
  //
  // Object Activation/Deactivation operations
  //

  // ObjectId activate_object(in Servant p_servant)
  public byte[] activate_object(Servant p_servant)
    {
      // get a unique object id
      byte[] objectId = generateUniqueId();

      activate_object_with_id(objectId, p_servant);
      
      return objectId;
    }

  // void activate_object_with_id(in ObjectId oid, 
  //                              in Servant p_servant)
  public void activate_object_with_id(byte[] oid,
				      Servant p_servant)
    {
      // create a container for the data
      ObjectMapInfo objectMapInfo =
	new ObjectMapInfo(oid, 
			  p_servant,
			  null,
			  true);

      // insert the new object
      objectMap_.put(objectMapInfo);

      // create an IOR for this object
      try
	{
	  this.id_to_reference(oid);
	}
      catch(Exception e)
	{
	}
   }

  // void deactivate_object(in ObjectId oid)
  //                raises (ObjectNotActive, WrongPolicy)
  // public void deactivate_object(byte[] oid)
  //  throws ObjectNotActive, WrongPolicy
  //  {
  //  }


  //
  // Reference Creation Operations
  //

  // Object create_reference(in CORBA::RepositoryId intf)
  //   raises(WrongPolicy)
  // public ORG.omg.CORBA.Object create_reference(String intf)
  //   throws WrongPolicy
  //   {
  //      
  //   }
  
  // Object ORG.omg.CORBA.Object create_reference_with_id(
  //            in ObjectId oid,
  //            in CORBA::RepositoryId intf)
  // public /* ORG.omg.CORBA.Object */ void
  //   create_reference_with_id(byte[] oid,
  // 		     String intf)
  //   {
  //   }


  //
  // Identity Mapping Operations
  //

  // ObjectId servant_to_id(in Servant p_servant)
  //   raises(ServantNotActive, WrongPolicy)
  // public byte[] servant_to_id(Servant p_servant)
  //   throws ServantNotActive, WrongPolicy
  //   {
  //   }

  // Object servant_to_reference(in Servant p_servant)
  //   raises(ServantNotActive, WrongPolicy)
  // public ORG.omg.CORBA.Object servant_to_reference(Servant p_servant)
  //   throws ServantNotActive, WrongPolicy
  //   {
  //   }
  
  // Servant reference_to_servant(in Object reference)
  //   raises(ObjectNotActive, WrongAdapter, WrongPolicy)
  // public Servant reference_to_servant(ORG.omg.CORBA.Object reference)
  //   throws ObjectNotActive, WrongAdapter, WrongPolicy
  //   {
  //   }
  
  // ObjectId reference_to_id(in Object reference)
  //   raises(WrongAdapter, WrongPolicy)
  // public byte[] reference_to_id(ORG.omg.CORBA.Object reference)
  //   throws WrongAdapter, WrongPolicy
  //   {
  //   }
  
  // Servant id_to_servant(in ObjectId oid)
  //   raises(ObjectNotActive, WrongPolicy)
  // public Servant id_to_servant(byte[] oid)
  //   throws ObjectNotActive, WrongPolicy
  //   {
  //   }

  // Object id_to_reference(in ObjectId oid)
  //   raises(ObjectNotActive, WrongPolicy)
  public ORG.omg.CORBA.Object id_to_reference(byte[] oid)
    throws ObjectNotActive, WrongPolicy
    {
      ObjectMapInfo objectMapInfo = objectMap_.get(oid);
      if(objectMapInfo == null)
	{
	  throw(new ObjectNotActive());
	}
      if(!objectMapInfo.isActive)
	{
	  throw(new ObjectNotActive());
	}

      if(objectMapInfo.reference != null)
	{
	  // System.out.println("objectMapInfo.reference != null");
	  return objectMapInfo.reference;
	}

      // create a new IOR for the object reference
      
      // create the TypeWriter
      ByteArrayOutputStream bodyStream = new ByteArrayOutputStream();
      TypeWriter bodyWriter = new CDRTypeWriter(bodyStream);

      // create the ProfileBody
      ProfileBody profileBody = new ProfileBody(host_,
						port_,
						oid);
      // encapsulate the ProfileBody
      try 
	{
	  profileBody.write(bodyWriter);
	  bodyWriter.flush();
	}
      catch(IOException ioEx)
	{
	  System.out.println("Fatal error encapsulating ProfileBody");
	  Runtime.getRuntime().exit(1);
	}
      
      byte[] profileEncapsulation = bodyStream.toByteArray();
                
      // put the profile body into the TaggedProfile
      TaggedProfile taggedProfile = 
	new TaggedProfile(TAG_INTERNET_IOP.value, 
			  profileEncapsulation);

      // Prepare the IOR
      TaggedProfile[] profiles = new TaggedProfile[1];
      profiles[0] = taggedProfile;
      IOR ior = new IOR(((Skeleton)objectMapInfo.servant).objType_,
			profiles);
                
      ((Skeleton)objectMapInfo.servant).setIOR(ior);
      return objectMapInfo.reference = (Skeleton)objectMapInfo.servant;
      // return objectMapInfo.reference = new ObjectReference(ior);
    }
  
  // private methods
  protected synchronized byte[] generateUniqueId()
    {
      int returnId = ++internalId_;
      byte[] aux = new byte[4];

      int i;
      for(i = 0; returnId != 0; i++)
	{
	  aux[i] = (byte)returnId;
	  returnId >>= 8;
	}
      
      byte[] result = new byte[i];
      for(i--; i>=0; i--)
	{
	  result[i] = aux[i];
	}
      
      return result;
    }
}
