/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT 0.01
 *
 */

package EDU.rutgers.CORBA.presentation;

// JDK
import java.io.InputStream;
import java.io.IOException;

// ACT
import ORG.omg.CORBA.MARSHAL;

import EDU.rutgers.act.presentation.TypeReader;

/**
 * @since ACT 0.01
 */
public class CDRTypeReader 
	extends TypeReader
{
    /**
     * The CDR transfer syntax used
     *
     * @since ACT 0.01
     */
    private static CDRTransferSyntax ts_ = CDRTransferSyntax.cdrTS_;
    /**
     * @param Reader actual reader to perform rading
     *
     * @since ACT 0.01
     */
	public CDRTypeReader(InputStream in) {
		super(in);
	}

    /**
     * @param int CDR alignemnt needed
     *
     * @since ACT 0.01
     */
	protected void setAlignment(int alignment) 
		throws IOException {
        	int gap = ts_.calAlignGap(index_, alignment); 
		index_ += gap;
		while(gap-- > 0)
		  in_.read();
	}

    /**
     * @since ACT 0.01
     */
    public boolean readBoolean()
		throws IOException, MARSHAL {
		index_ ++;
		return (in_.read() != 0); 
	}

    /**
     * @since ACT 0.01
     */
    public char readChar() 
        throws IOException {
	index_ ++;
        return (char)in_.read();
	}

    /**
     * @since ACT 0.01
     */
    public byte readOctet()
		throws IOException {
		index_ ++;
		return (byte)in_.read();
	}

    /**
     * @since ACT 0.01
     */
    public short readShort()
		throws IOException {
	setAlignment(2);
	index_ += 2;
	int b1 = in_.read();
	int b2 = in_.read();
        if (littleEndian_) {
            return (short)(b1 | (b2 << 8)); 
        }
        else {
            return (short)(b2 | (b1 << 8)); 
        }
	}

    /**
     * @since ACT 0.01
     */
    public short readUShort() 
        throws IOException {
	// the same code as readShort()
	setAlignment(2);
	index_ += 2;
	int b1 = in_.read();
	int b2 = in_.read();
        if (littleEndian_) {
            return (short)(b1 | (b2 << 8)); 
        }
        else {
            return (short)(b2 | (b1 << 8)); 
        }
	}

    /**
     * @since ACT 0.01
     */
    public int readLong()
		throws IOException {
        setAlignment(4);
	index_ += 4;
	int b1 = in_.read();
	int b2 = in_.read();
	int b3 = in_.read();
	int b4 = in_.read();

        if (littleEndian_)
	    return ((((((b4 << 8) | b3) << 8) | b2) << 8) | b1);
        else
	    return (((((b1 << 8) | b2) << 8) | b3) << 8) | b4;
	}

    /**
     * @since ACT 0.01
     */
    public int readULong()
		throws IOException {
	// same code as in readLong()
        setAlignment(4);
	index_ += 4;
	int b1 = in_.read();
	int b2 = in_.read();
	int b3 = in_.read();
	int b4 = in_.read();

        if (littleEndian_)
	    return ((((((b4 << 8) | b3) << 8) | b2) << 8) | b1);
        else
	    return (((((b1 << 8) | b2) << 8) | b3) << 8) | b4;
	}

    /**
     * @since ACT 0.01
     */
    public long readLongLong()
		throws IOException {
        setAlignment(8);
	index_ += 8;
	long b1 = in_.read();
	long b2 = in_.read();
	long b3 = in_.read();
	long b4 = in_.read();
	long b5 = in_.read();
	long b6 = in_.read();
	long b7 = in_.read();
	long b8 = in_.read();

        if (littleEndian_)
	    return ((((((((((((((b8 << 8) | b7) << 8) | b6) << 8) | b5)
	<< 8) | b4) << 8) | b3) << 8) | b2) << 8) | b1);
        else
	    return ((((((((((((((b1 << 8) | b2) << 8) | b3) << 8) | b4)
	<< 8) | b5) << 8) | b6) << 8) | b7) << 8) | b8);
	}

    /**
     * @since ACT 0.01
     */
    public long readULongLong()
		throws IOException {
	// same code as in readLongLong
        setAlignment(8);
	index_ += 8;
	long b1 = in_.read();
	long b2 = in_.read();
	long b3 = in_.read();
	long b4 = in_.read();
	long b5 = in_.read();
	long b6 = in_.read();
	long b7 = in_.read();
	long b8 = in_.read();

        if (littleEndian_)
	    return ((((((((((((((b8 << 8) | b7) << 8) | b6) << 8) | b5)
	<< 8) | b4) << 8) | b3) << 8) | b2) << 8) | b1);
        else
	    return ((((((((((((((b1 << 8) | b2) << 8) | b3) << 8) | b4)
	<< 8) | b5) << 8) | b6) << 8) | b7) << 8) | b8);
	}

    /**
     * @since ACT 0.01
     */
    public float readFloat()
		throws IOException {
        return Float.intBitsToFloat(readLong()); 
	}

    /**
     * @since ACT 0.01
     */
    public double readDouble()
		throws IOException {
        return Double.longBitsToDouble(readLongLong()); 
	}

    /**
     * @since ACT 0.01
     */
    public String readString()
		throws IOException {
        int len = readLong(); 
        byte[] ba= new byte[len]; 
	index_ += len;
	in_.read(ba, 0, len);
        String str = new String(ba, 0, len -1);
        return str; 
	}
}
