/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT 0.01
 *
 */

package EDU.rutgers.CORBA.presentation;

// JDK
import java.io.OutputStream;
import java.io.IOException;

// ACT
import ORG.omg.CORBA.CompletionStatus;
import ORG.omg.CORBA.DATA_CONVERSION;

import EDU.rutgers.act.presentation.TypeWriter;

/**
 * @since ACT 0.01
 */
public class CDRTypeWriter extends TypeWriter
{
    /**
     * The CDR transfer syntax used
     * @since ACT 0.01
     */
    private static CDRTransferSyntax ts_ = CDRTransferSyntax.cdrTS_;

    /**
     * @param Writer actual writer to perform writing
     *
     * @since ACT 0.01
     */
    public CDRTypeWriter(OutputStream out) {
		super(out);
//System.out.println("CDRTypwWriter: created a CDRTypeWrite");
    }

    /**
     * @param int CDR alignemnt needed
     *
     * @since ACT 0.01
     */
 	protected void setAlignment(int alignment)
		throws IOException {
        	int gap = ts_.calAlignGap(index_, alignment); 
		index_ += gap;
	        while(gap-- > 0)
			out_.write(0);
	}

    /**
     * @since ACT 0.01
     */
	public void writeBoolean(boolean b)
		throws IOException {
	index_++;
	out_.write(b?1:0);
	}

    /**
     * @since ACT 0.01
     */
	public void writeChar(char c)
        throws IOException {
		index_++;
		out_.write((int)c);
	}

    /**
     * @since ACT 0.01
     */
	public void writeOctet(byte b)
		throws IOException {
		index_ ++;
		out_.write(b);
	}

    /**
     * @since ACT 0.01
     */
	public void writeShort(short s)
	throws IOException {
	setAlignment(2);
	index_ += 2;
	byte b1 = (byte)(s & 0xFF);
	byte b2 = (byte)((s >> 8) & 0xFF);
        if (littleEndian_) {
			out_.write(s);
			out_.write(s >> 8);
        }
	else{
			out_.write(s >> 8);
			out_.write(s);
	}
	}

    /**
     * @since ACT 0.01
     */
    public void writeUShort(short s)
		throws IOException {
	// same code as in writeShort()
	setAlignment(2);
	index_ += 2;
	byte b1 = (byte)(s & 0xFF);
	byte b2 = (byte)((s >> 8) & 0xFF);
        if (littleEndian_) {
			out_.write(s);
			out_.write(s >> 8);
        }
	else{
			out_.write(s >> 8);
			out_.write(s);
	}
	}

    /**
     * @since ACT 0.01
     */
	public void writeLong(int l)
		throws IOException {
		setAlignment(4);
		index_ += 4;
		int b1 = l;
		int b2 = b1 >> 8;
		int b3 = b2 >> 8;
		int b4 = b3 >> 8;

        if (littleEndian_) {
			out_.write(b1);
			out_.write(b2);
			out_.write(b3);
			out_.write(b4);
		}
	else{
		out_.write(b4);
		out_.write(b3);
		out_.write(b2);
		out_.write(b1);
	}
	}

    /**
     * @since ACT 0.01
     */
    public void writeULong(int ul)
		throws IOException {
	// same code as in writeLong()
	setAlignment(4);
	index_ += 4;
	int b1 = ul;
	int b2 = b1 >> 8;
	int b3 = b2 >> 8;
	int b4 = b3 >> 8;

        if (littleEndian_) {
			out_.write(b1);
			out_.write(b2);
			out_.write(b3);
			out_.write(b4);
		}
	else{
		out_.write(b4);
		out_.write(b3);
		out_.write(b2);
		out_.write(b1);
	}
	}

    /**
     * @since ACT 0.01
     */
	public void writeLongLong(long l)
		throws IOException {

	setAlignment(8);
	index_ += 8;
	int b1 = (int)l;
	int b2 = (int)(l >>= 8);
	int b3 = (int)(l >>= 8);
	int b4 = (int)(l >>= 8);
	int b5 = (int)(l >>= 8);
	int b6 = (int)(l >>= 8);
	int b7 = (int)(l >>= 8);
	int b8 = (int)(l >>= 8);


        if (littleEndian_) {
			out_.write(b1);
			out_.write(b2);
			out_.write(b3);
			out_.write(b4);
			out_.write(b5);
			out_.write(b6);
			out_.write(b7);
			out_.write(b8);
		}
	else{
		out_.write(b8);
		out_.write(b7);
		out_.write(b6);
		out_.write(b5);
		out_.write(b4);
		out_.write(b3);
		out_.write(b2);
		out_.write(b1);
	}
	}

    /**
     * @since ACT 0.01
     */
    public void writeULongLong(long l)
		throws IOException {
	// same code as in writeLongLong()
	setAlignment(8);
	index_ += 8;
	int b1 = (int)l;
	int b2 = (int)(l >>= 8);
	int b3 = (int)(l >>= 8);
	int b4 = (int)(l >>= 8);
	int b5 = (int)(l >>= 8);
	int b6 = (int)(l >>= 8);
	int b7 = (int)(l >>= 8);
	int b8 = (int)(l >>= 8);

        if (littleEndian_) {
			out_.write(b1);
			out_.write(b2);
			out_.write(b3);
			out_.write(b4);
			out_.write(b5);
			out_.write(b6);
			out_.write(b7);
			out_.write(b8);
		}
	else{
		out_.write(b8);
		out_.write(b7);
		out_.write(b6);
		out_.write(b5);
		out_.write(b4);
		out_.write(b3);
		out_.write(b2);
		out_.write(b1);
	}
	}

    /**
     * @since ACT 0.01
     */
	public void writeFloat(float f)
		throws IOException {
		writeLong(Float.floatToIntBits(f));
	}

    /**
     * @since ACT 0.01
     */
    public void writeDouble(double d)
		throws IOException {
		writeLongLong(Double.doubleToLongBits(d));
	}

    /**
     * @since ACT 0.01
     */
	public void writeString(String str)
		throws IOException {
		byte[] b = str.getBytes();
		int len = b.length;
		writeLong(len + 1);
		index_ += len + 1;
		out_.write(b, 0, len);
		out_.write(0);
	}
}