// for emacs: -*- C++ -*-

// last update: 07/30/97
// 03/12/98 added verbose mode

package EDU.rutgers.act.clntsrv.threadengine;

import EDU.rutgers.act.protocol.*;
import EDU.rutgers.util.Verbose;

/**
 * This class implements the ThreadPerRequest
 * policy.  It is used in combination with
 * TPCConnectionFactory.  If used with
 * SEQConnectionFactory it yields in a strange
 * multi-threading model where the server
 * accepts only one connection at a time,
 * but all the requests on that connection
 * are processed in parallel.  This combination
 * shouldn't be used, unless there is a real
 * need for this weird model.
 **/
public class TPRRequestHandlerFactory implements RequestHandlerFactory
{ 
  Service service_;

  public TPRRequestHandlerFactory(Service service)
    {
      service_ = service;
              if(Verbose.verbose)
		{
		  Verbose.print("TPRRequestHandlerFactory",
				"TPRRequestHandlerFactory",
				"created");
		}
    }
  

  // public methods
  public void createRequestHandler(Message request,
				   Sender serverSender)
    {
              if(Verbose.verbose)
		{
		  Verbose.print("TPRRequestHandlerFactory",
				"createRequestHandler",
				"creating the RequestHandler");
		}
      RequestHandler requestHandler = 
	new RequestHandler(service_,
			   request, 
			   serverSender);

              if(Verbose.verbose)
		{
		  Verbose.print("TPRRequestHandlerFactory",
				"createRequestHandler",
				"launching the RequestHandler");
		}
      new Thread(requestHandler).start();
              if(Verbose.verbose)
		{
		  Verbose.print("TPRRequestHandlerFactory",
				"createRequestHandler",
				"launched");
		}
    }
}
