/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.act.presentation;

//import java.io.Reader;
import java.io.FilterReader;
import java.io.InputStream;
//import java.io.FilterInputStream;

import ORG.omg.CORBA.BAD_PARAM;
import ORG.omg.CORBA.MARSHAL;

/**
 * Read primitives and primitive arrays from stream based on sepcific transfer syntax
 *
 * @see TransferSyntax
 * @see TypeWriter
 *
 * @since ACT 0.1
 */
public abstract class TypeReader 
//	extends FilterReader
{
    /**
     * @param Reader the actual reader used to perform reading
     *
     * @since ACT 0.1
     */
	public InputStream in_;
    protected TypeReader(InputStream in) {
//		super(in);
		in_ = in;
		index_ = 0;
    }

    /**
     * Index to be aligned with
     *
     * @since ACT 0.1
     */
    protected int index_;

    /**
     * Byte order : true for little endian, false for big endian
     *
     * @since ACT 0.1
     */
	protected boolean littleEndian_;

    /**
     * @param int align index
     *
     * @see #index_
     *
     * @since ACT 0.1
     */
	public void setIndex(int index) { 
		index_ = index; 
	}

    /**
     * @return int align index
     *
     * @see #index_
     *
     * @since ACT 0.1
     */
    public int getIndex() {
        return index_;
    }

    /**
     * @param boolean true for little endian, false for big endian
     *
     * @see littleEndian
     *
     * @since ACT 0.1
     */
	public void setByteOrder(boolean littleEndian) {
		littleEndian_ = littleEndian;
	}

    /**
     * @return boolean true for little endian, false for big endian
     *
     * @see littleEndian
     *
     * @since ACT 0.1
     */
	public boolean getByteOrder() {
		return littleEndian_;
	}

    /**
     * @exception java.io.IOException if any IO error occurs
     * @exception MARSHAL if the octet read is not 1 or 0
     *
     * @since ACT 0.1
     */
    public abstract boolean readBoolean()
		throws java.io.IOException, MARSHAL;

    /**
     * @return char not UNICODE
     *
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract char readChar()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract byte readOctet()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract short readShort()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract short readUShort()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract int readLong()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract int readULong()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract long readLongLong()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract long readULongLong()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract float readFloat()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract double readDouble()
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract String readString()
		throws java.io.IOException;

    /**
     * @param boolean[] boolean array
     * @param int offset of the array to begin to read in
     * @param int number of boolean to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readBooleanArray(boolean[] ba, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (ba== null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= ba.length) {
                break;
            }
            ba[index] = readBoolean(); 
        }
	}

    /**
     * @param char[] char array (no UNICODE)
     * @param int offset of the array to begin to read in
     * @param int number of char to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readCharArray(char[] ca, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (ca == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= ca.length) {
                break;
            }
            ca[index] = readChar(); 
        }
	}

    /**
     * @param byte[] byte array (convert to unsigned)
     * @param int offset of the array to begin to read in
     * @param int number of byte to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
	public void readOctetArray(byte[] oa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (oa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= oa.length) {
                break;
            }
            oa[index] = readOctet(); 
        }
	}

    /**
     * @param short[] short array 
     * @param int offset of the array to begin to read in
     * @param int number of short to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readShortArray(short[] sa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (sa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= sa.length) {
                break;
            }
            sa[index] = readShort(); 
        }
	}

    /**
     * @param short[] short array
     * @param int offset of the array to begin to read in
     * @param int number of short to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readUShortArray(short[] usa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (usa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= usa.length) {
                break;
            }
            usa[index] = readUShort(); 
        }
	}

    /**
     * @param int[] int array
     * @param int offset of the array to begin to read in
     * @param int number of int to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readLongArray(int[] la, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (la == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= la.length) {
                break;
            }
            la[index] = readLong(); 
        }
	}

    /**
     * @param int[] int array
     * @param int offset of the array to begin to read in
     * @param int number of int to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readULongArray(int[] ula, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (ula== null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= ula.length) {
                break;
            }
            ula[index] = readULong(); 
        }
	}

    /**
     * @param long[] long array
     * @param int offset of the array to begin to read in
     * @param int number of long to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readLongLongArray(long[] lla, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (lla == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= lla.length) {
                break;
            }
            lla[index] = readLongLong(); 
        }
	}

    /**
     * @param long[] long array
     * @param int offset of the array to begin to read in
     * @param int number of long to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readULongLongArray(long[] ulla, int offset, int len)
        throws java.io.IOException, BAD_PARAM {
        if (ulla== null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= ulla.length) {
                break;
            }
            ulla[index]= readULongLong(); 
        }
	}

    /**
     * @param float[] float array
     * @param int offset of the array to begin to read in
     * @param int number of float to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readFloatArray(float[] fa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (fa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= fa.length) {
                break;
            }
            fa[index]= readFloat(); 
        }
	}

    /**
     * @param double[] double array
     * @param int offset of the array to begin to read in
     * @param int number of double to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readDoubleArray(double[] da, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (da== null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= da.length) {
                break;
            }
            da[index] = readDouble(); 
        }
	}

    /**
     * @param String[] String array (no UNICODE)
     * @param int offset of the array to begin to read in
     * @param int number of String to read in
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void readStringArray(String[] stra, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (stra== null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index= offset; index < (offset + len); index++) {
            if (index >= stra.length) {
                break;
            }
            stra[index] = readString(); 
        }
	}
}
