// for emacs only: -*- C++ -*-

// last modification 09/17/97

// Aug. 14, 1997
// Modified by Weicong Wang

// 03/12/98 added verbose mode

package EDU.rutgers.act.protocol.tcp;

import java.net.*;
import java.io.IOException;

import EDU.rutgers.util.Verbose;
import EDU.rutgers.act.presentation.TransferSyntaxFactory;
import EDU.rutgers.act.protocol.*;

public class ProtocolServer implements Server
{
  // data members

  protected FSMDataFactory fsmDataFactory_;
  protected MessageFactory messageFactory_;
  protected ServerSocket serverSocket_;
  protected short port_;
  protected TransferSyntaxFactory tsf_;
  protected SocketHandler sh_;
  protected int fsmId_;
  
  // constructors

  public ProtocolServer(FSMDataFactory fsmDataFactory,
			MessageFactory messageFactory,
			TransferSyntaxFactory dataTsf_,
			int listeningPort)
        throws IOException
    {
      fsmDataFactory_ = fsmDataFactory;
      messageFactory_ = messageFactory;
      tsf_ = dataTsf_;

              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"ProtocolServer",
				"scanning for free ports to listen starting with " + 
				listeningPort);
		}
      // find the first free port
      port_ = 0;
      for(int i = listeningPort; i < listeningPort + 10000; i++)
	{
	  try
	    {
	              if(Verbose.verbose)
			{
			  Verbose.print("ProtocolServer",
					"ProtocolServer",
					"trying to open ServerSocket on port " + i);
			}
	      serverSocket_ = new ServerSocket((short)i);
              serverSocket_.setSoTimeout(0);
	              if(Verbose.verbose)
			{
			  Verbose.print("ProtocolServer",
					"ProtocolServer",
					"opened the ServerSocket on port " + i);
			}
	      port_ = (short)i;
	      i = listeningPort + 10000;
	    }
	  catch(Exception e) // unsuccessful
	    {
	              if(Verbose.verbose)
			{
			  Verbose.print("ProtocolServer",
					"ProtocolServer",
					"unsuccessful attempt");
			}
	    }
	}
      if(port_ == 0)
	throw new IOException("no free ports found in range " + listeningPort + " - " + (listeningPort + 10000));
      System.out.println("@ProtocolServer: listening port is: " + port_);

              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"ProtocolServer",
				"creating a SocketHandler with args: " +
				messageFactory_ + " and " +
				tsf_ + " and " + "null");
		}
      sh_ = new SocketHandler(messageFactory_, tsf_, null);
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"ProtocolServer",
				"created a SocketHandler:" + sh_);
		}
      fsmId_ = 0;
    }

  // methods

  public synchronized Protocol accept() throws java.io.IOException
    {
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"going to accept on port " + port_);
		}
      Socket socket = serverSocket_.accept();
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"accepted a new connection on port " + port_);
		}
      socket.setSoTimeout(0);
      FSMData fsmData = fsmDataFactory_.createFSMData();
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"notifying the SocketHandler about this socket");
		}
      sh_.putSocket(socket, --fsmId_);
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"creating an FSM with args: " +
				fsmData + " and " + sh_ + " and " + fsmId_);
		}
      FSM fsm = new FSM(fsmData, sh_, fsmId_);
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"created the FSM: " + fsm);
		}
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"creating a Protocol object");
		}
      Protocol protocol = new Protocol(new ServerReceiver(fsm),
				       new ServerSender(fsm));
              if(Verbose.verbose)
		{
		  Verbose.print("ProtocolServer",
				"accept",
				"created the Protocol object: " + protocol);
		}
      return protocol;
    }

  public short getPort()
    {
      return port_;
    }
}
