/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

// last modification: 01/23/98
// 03/29/98 added verbose mode

package EDU.rutgers.act.protocol.tcp;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import EDU.rutgers.act.protocol.*;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;
import EDU.rutgers.act.presentation.TransferSyntaxFactory;

import EDU.rutgers.act.endpoint.Endpoint;
import EDU.rutgers.act.protocol.*;
import EDU.rutgers.util.Verbose;

public class SocketHandler
{
  // data members
  
  protected MessageFactory factory_;
  protected TransferSyntaxFactory tsf_;
  protected Hashtable sockets_, fsms_;

  // constructors
  public SocketHandler(MessageFactory factory, 
			TransferSyntaxFactory dataTsf, 
			Socket socket)
    {
      sockets_ = new Hashtable();
      fsms_ = new Hashtable();

      factory_ = factory;
      tsf_ = dataTsf;
      
      if(socket != null)
	{
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "SocketHandler",
				      "adding socket: " + socket);
		      }
	  putSocket(socket);
	}
    }
  

  // methods

  // get the unique key for the IP and port
  String toKey(InetAddress ia, int port)
    {
      byte[] p = new byte[6];
      System.arraycopy(ia.getAddress(), 0, p, 0, 4);
      
      p[4] = (byte)port;
      p[5] = (byte)(port >> 8);
      
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "toKey",
				      "key=" + new String(p));
		      }
      return new String(p);
    }
  
  SocketInfo addSocket(Socket socket)
    {
      String key = toKey(socket.getInetAddress(), socket.getPort());
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "addSocket",
				      "put socket: " + socket);
		      }
      SocketInfo si = new SocketInfo();
      si.socket_ = socket;
      try
	{
	  si.out_ = new BufferedOutputStream(socket.getOutputStream());
	}
      catch(IOException e) {}
      sockets_.put(key, si);
      return si;
    }
  
  SocketInfo putSocket(Socket socket, int fsmId)
    {
      // add the socket in the socket table
      SocketInfo si = addSocket(socket);

      // add the fsm entry in the fsm table
      FSMMapInfo info = new FSMMapInfo(fsmId,
			    (Message)null,
			    null,
			    true);
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "putSocket",
				      "add entry to fsms_ with id: " + fsmId);
		      }
      fsms_.put(new Integer(info.id_), info);

      // launch the reader thread for this socket
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "putSocket",
				      "launching the SocketReader thread");
		      }
      new SocketReader(si, this, fsms_, tsf_, factory_, fsmId);
      return si;
    }
  
  SocketInfo putSocket(Socket socket)
    {
      SocketInfo si = addSocket(socket);
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "putSocket",
				      "launching the SocketReader thread");
		      }
      // launch the reader thread for this socket
      new SocketReader(si, this, fsms_, tsf_, factory_, 0);
      return si;
    }

  SocketInfo getSocket(Endpoint ep)
    throws java.net.UnknownHostException, java.io.IOException
    {
      String key = toKey(InetAddress.getByName(ep.hostName_), ep.port_);
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "getSocket",
				      "found socket: " + 
				      (SocketInfo)sockets_.get(key));
		      }
      return (SocketInfo)sockets_.get(key);
    }

  void removeSocket(Socket socket)
    {
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "removeSocket",
				      "removing socket: " + 
				      socket);
		      }
      sockets_.remove(toKey(socket.getInetAddress(), socket.getPort()));
      try
	{
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "removeSocket",
				      "trying to close socket");
		      }
	  socket.close();
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "removeSocket",
				      "closed");
		      }
	}
      catch(java.io.IOException e)
	{
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "removeSocket",
				      "exception: " + e);
		      }
	}
    }

  public void sendMessage(Message message, int fsmId)
    throws java.net.UnknownHostException, java.io.IOException
    {
      SocketInfo socket = null;
      if(fsms_ == null)
	{
	  System.out.println("SocketHandler.sendMessage: null fsms_!");
	}
      FSMMapInfo info = (FSMMapInfo)fsms_.get(new Integer(message.getTransactionId()));
      Endpoint ep;
      
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "sendMessage",
				      "info=" + info);
		      }
      // first get a valid socket for sending the message
      //
      // does the current message specify a destination?
      if((ep = message.getEndpoint()) == null)
	{
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "sendMessage",
				      "no destination in the message");
		      }
	  // if not then do we have a previous record on this transaction?
	  if(info == null)
	    {
	      // try the alternate record, of the FSM...
	      info = (FSMMapInfo)fsms_.get(new Integer(fsmId));
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "sendMessage",
				      "info=" + info);
		      }
	    }
	  

	  // do we have a previous alternate record on this transaction?
	  if(info == null)
	    {
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "sendMessage",
				      "don't know where to send the message!");
		      }
	      //if not, we can't find the destination!
	      throw new java.net.UnknownHostException("No endpoint specified in message: " + message);
	    }
	  else
	    // if we have a record then is there a socket associated with it?
	    if((socket = info.socket_) == null)
	      {
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "sendMessage",
				      "don't know where to send the message!");
		      }
		// if not, we can't find the destination
		throw new java.net.UnknownHostException("No endpoint specified in message: " + message);
	      }
	}
      else // we have a specific Endpoint to send to
	{
                    if(Verbose.verbose)
		      {
			Verbose.print("SocketHandler",
				      "sendMessage",
				      "checking if socket exists");
		      }
	  // check if the socket is already open
	  if((socket = getSocket(ep)) == null)
	    {
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "it doesn't, create a new one");
			  }
	      // time to create a new socket!
	      // System.out.println("EDU.rutgers.act.protocol.SocketHandler.sendMessage(): Creating a new socket!");
	      Socket s = null;
	      try
		{
		  s = new Socket(ep.hostName_, ep.port_);
		  s.setSoTimeout(0);
		}
	      catch(UnknownHostException e)
		{
		  System.out.println("@SocketHandler.sendMessage(): unknown host: <" + ep.hostName_ + ">\n" + e);
		}
	      catch(IOException e)
		{
		  System.out.println("@SocketHandler.sendMessage(): IO  exception :" + e);
		}
	      
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "add the socket to the hash table");
			  }
	      socket = putSocket(s);
	    }
	}
      

                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "info=" + info);
			  }
      // now add the needed info in the FSMMapInfo entry
      if(info == null)
	{
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "creating info");
			  }
	  info = new FSMMapInfo(message.getTransactionId(),
				(Message)null,
				socket,
				true);
	  
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "add entry to fsms_ with id: " + info.id_);
			  }
	  fsms_.put(new Integer(info.id_), info);
	}
      else
	{
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "info existent, set msg=null sock=" +
					  socket + " wait=true");
			  }
	  info.message_ = null;
	  info.socket_ = socket;
	  info.waiting_ = true;
	}
      

                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "sendMessage",
					  "send message");
			  }
      // we can now safely send the message
      TypeWriter typeWriter = tsf_.createTypeWriter(socket.out_);
      message.write(typeWriter);
      typeWriter.flush();
    }

  public Message recvMessage(int id) throws java.io.IOException
    {
      Integer key = new Integer(id);
      
      FSMMapInfo info = (FSMMapInfo)fsms_.get(key);

                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "recvMessage",
					  "info=" + info);
			  }
      if(info == null)
	{
	  throw new java.io.IOException("No answer expected for TRANSACTION ID " + id);
	}

                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "recvMessage",
					  "getting the message");
			  }
      Message m = info.getMessage();

                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "recvMessage",
					  "msg id=" + id);
			  }
      if(id >= 0)
	{
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "recvMessage",
					  "remove FSM: " + id);
			  }
	  fsms_.remove(key);
	}

                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "recvMessage",
					  "returning message: " + m);
			  }
      return m;
    }

  public void finalize() throws IOException
    {
                        if(Verbose.verbose)
			  {
			    Verbose.print("SocketHandler",
					  "finalize",
					  "finalizing the SocketHandler");
			  }
      factory_ = null;
      sockets_ = null;
      fsms_ = null;
    }
}
