package EDU.rutgers.cbus.clntsrv;

import java.io.IOException;
import java.net.Socket;
import java.net.InetAddress;

import EDU.rutgers.act.endpoint.*;
import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.presentation.*;
import EDU.rutgers.act.clntsrv.threadengine.*;
import EDU.rutgers.cbus.protocol.messages.*;
import EDU.rutgers.cbus.protocol.actions.*;
import EDU.rutgers.cbus.presentation.*;

public class Communicator extends Thread
{
  // protected static boolean created_ = false;
  /*
  private static final Communicator communicator_ = 
    new Communicator(true); // true = multicast
    */
  protected static final String LOG_NAME = "CBLOG";
  protected static final String MADDR = "224.1.2.3"; // multicast addr
  protected final short DEFAULT_PORT = 6000;
  protected final int TTL = 4;

  protected String host_; // local host
  protected short port_;  // local port

  protected OA oa_ = null;
  protected EndpointManager em_ = null;

  protected Sender clientSender_;
  protected Acceptor acceptor_;
  protected Server protSrv_;

  protected boolean mcast_;

/*
  public static final Communicator getCommunicator()
    {
      while(!created_)
	try { Thread.sleep(100); } catch(Exception e) {} 
      return communicator_;
    }
*/
  public Communicator(boolean mcast)
    {
      // is it a multicast communicator?
      mcast_ = mcast;

      // retain host and port
      host_ = null;
      try {
	host_ = InetAddress.getLocalHost().getHostAddress();
	System.out.println("@Communicator: local host is <" + host_ + ">");
      }
      catch(java.net.UnknownHostException e)
	{
	  System.out.println("@Communicator.java: error finding local host: "
			     + e);
	}
      port_ = (short)(DEFAULT_PORT + 
		      (mcast_?0:(System.currentTimeMillis() & 0xFF)));
      
      //
      // creation of the protocol layer --- client side
      //

      FSMDataFactory clntDataFactory = new CBusClientDataFactory();
      MessageFactory clntMsgFactory = new CBusEventFactory();
      TransferSyntaxFactory clntTSF = new CBusTransferSyntaxFactory();

      if(mcast_)
	{
	  clientSender_ = 
	      new EDU.rutgers.act.protocol.mcast.ClientSender(
                  clntTSF, TTL);
	}
      else
	clientSender_ = 
	  new EDU.rutgers.act.protocol.tcp.ClientSender(clntDataFactory,
					 clntMsgFactory,
					 clntTSF);
      
      // end creation of protocol layer --- client side

      // create the endpoint manager
      em_ = new EndpointManager(clientSender_, mcast_);

      // create the object adaptor
      oa_ = new OA();
      // created_ = true;
    }

  public void startAcceptingConnections(String host, int port)
    throws IOException
    {
      //
      // creation of the protocol layer --- server side

      FSMDataFactory srvFactory = new CBusServerDataFactory();
      MessageFactory srvMsgFactory = new CBusEventFactory();
      TransferSyntaxFactory clntTSF = new CBusTransferSyntaxFactory();

      try
	{
	  if(mcast_)
	    {
	      Endpoint ep = new Endpoint(host, port);
	      ((EDU.rutgers.act.protocol.mcast.ClientSender)
		clientSender_).setEndpoint(ep);

	      protSrv_ = 
		new EDU.rutgers.act.protocol.mcast.ProtocolServer(
		  srvMsgFactory,
                  clntTSF,
                  (EDU.rutgers.act.protocol.mcast.ClientSender)clientSender_); 
	    }
	  else
	    protSrv_ = 
	      new EDU.rutgers.act.protocol.tcp.ProtocolServer(srvFactory,
		  			  srvMsgFactory,
					  clntTSF,
					  port);
	  port_ = protSrv_.getPort();
	}
      catch(java.io.IOException e)
	{
	  System.err.println(
	    "@Communicator.java: error creating ProtocolServer " + e);
	  System.exit(1);
	}

      // end creation of protocol layer --- server side

      // set local host and port on the Object Adaptor
      oa_.setLocalAddress(host_, port_);

      // Thread Engine creation
      // obtaining a Sequential RequestHandlerFactory
      RequestHandlerFactory requestHandlerFactory = (RequestHandlerFactory)
	new SEQRequestHandlerFactory(oa_);

      // obtaining a Thread-Per-Session ConnectionFactory
      ConnectionFactory connectionFactory = (ConnectionFactory)
	new TPCConnectionFactory(requestHandlerFactory);

      // obtaining the Acceptor (Thread Per Session policy)
      acceptor_ = new Acceptor(connectionFactory, protSrv_);

      // end of Thread Engine creation
      this.start();

      // in case of multicast start sending events to the multicast 
      // address/port
      if(mcast_)
	addClient(host, port);
    }

  public void startAcceptingConnections(int port) throws IOException
    {
      startAcceptingConnections(MADDR, port);
    }
  
  public int startAcceptingConnections() throws IOException
    {
      startAcceptingConnections(MADDR, port_);
      return port_;
    }

  public void replicateEvent(CBusEvent event)
    {
      em_.replicateEvent(event);
    }
  
  public boolean addClient(String host, int port)
    {
      em_.addEndpoint(new Endpoint(host, port));
      return true;
    }

  public void removeClient(String host, int port)
    {
      em_.removeEndpoint(new Endpoint(host, port));
    }

  public int connect(Servant servant)
    {
      return oa_.put(servant);
    }

  public void logEvents(boolean logem)
    {
/*
      // manufacture an artificial name
      String logFileName = LOG_NAME + (System.currentTimeMillis() & 1023);
      logEvents(logem, logFileName);
*/
    }

  // starts queueing events, for logging
  public synchronized void logEvents(boolean logem, String logFileName)
    {
/*
      // start logging
      if(logem && !logging_)
	{
	  logging_ = true;
	  try { logQ_ = new EventQ(512);} catch(QException qe) {} 
	  eventLogger_ = new EventLogger(logQ_, logFileName);
	}

      // stop logging
      if(!logem && logging_)
	{
	  logging_ = false;
	  logQ_.enqueue(null);
	}

      // Unfortunately the queue and event logger
      // will not be garbage collected, since their pointers
      // are not released.  They are collected if
      // the method is called again, with logem = true.
*/
    }

  // starts playing events, from a log file
  public synchronized void playEvents(boolean playem,
				      String logFileName,
				      int delay)
    {
/*
      // start playing
      if(playem && !playing_)
	{
	  playing_ = true;
	  try { alternateQ_ = new EventQ(512);} catch(QException qe) {} 
	  currentQ_.enqueue(null);
	  eventPlayer_ = new EventPlayer(alternateQ_, logFileName, delay);
	}

      // stop playing
      if(!playem && playing_)
	{
	  playing_ = false;
	  alternateQ_ = q_;
	  currentQ_.enqueue(null);
	  eventPlayer_.stopPlaying();
	  eventPlayer_ = null;
	}
*/
    }

  public void run()
    {
      try
	{
	  acceptor_.run();
	}
      catch(java.io.IOException e)
	{
	  System.err.println("@Communicator: error running the acceptor " + e);
	  System.exit(1);
	}
    }

  public void close()
    {
    }

}
