/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		CBus 0.01
 *
 */

package EDU.rutgers.cbus.presentation;

// JDK
import java.io.InputStream;
import java.io.IOException;

// ACT
import EDU.rutgers.act.presentation.TypeReader;

/**
 * @since CBus 0.01
 */
public class CBusTypeReader 
	extends TypeReader
{
  /**
   * @param Reader actual reader to perform rading
   *
   * @since CBus 0.01
   */
  public CBusTypeReader(InputStream in)
    {
      super(in);
    }

  /**
   * @since CBus 0.01
   */
  public boolean readBoolean() throws IOException
    {
      return (in_.read() != 0);
    }

  /**
   * @since CBus 0.01
   */
  public char readChar() throws IOException 
    {
      return (char)in_.read();
    }

  /**
   * @since CBus 0.01
   */
  public byte readOctet() throws IOException
    {
      return (byte)in_.read();
    }

  /**
   * @since CBus 0.01
   */
  public short readShort() throws IOException
    {
      int b1 = in_.read();
      int b2 = in_.read();
      return (short)(b1 | (b2 << 8)); 
    }

  /**
   * @since CBus 0.01
   */
  public short readUShort() throws IOException
    {
      // identical with readShort
      int b1 = in_.read();
      int b2 = in_.read();
      return (short)(b1 | (b2 << 8)); 
    }

  /**
   * @since CBus 0.01
   */
  public int readLong() throws IOException
    {
      int b1 = in_.read();
      int b2 = in_.read();
      int b3 = in_.read();
      int b4 = in_.read();
      return ((((((b4 << 8) | b3) << 8) << b2) << 8) | b1);
    }

    /**
     * @since CBus 0.01
     */
  public int readULong() throws IOException 
    {
      // identical with readLong
      int b1 = in_.read();
      int b2 = in_.read();
      int b3 = in_.read();
      int b4 = in_.read();
      return ((((((b4 << 8) | b3) << 8) | b2) << 8) | b1);
    }

  /**
   * @since CBus 0.01
   */
  public long readLongLong() throws IOException 
    {
      long low = (long)readLong();
      long high = (long)readLong();
      return (high << 32) | low;
    }

  /**
   * @since CBus 0.01
   */
  public long readULongLong() throws IOException 
    {
      return (long)readLongLong();
    }

  /**
   * @since CBus 0.01
   */
  public float readFloat() throws IOException 
    {
      return Float.intBitsToFloat(readLong()); 
    }

  /**
   * @since CBus 0.01
   */
  public double readDouble() throws IOException 
    {
      return Double.longBitsToDouble(readLongLong()); 
    }

  /**
   * @since CBus 0.01
   */
  public String readString() throws IOException
    {
      int len = readLong(); 
      byte[] ba= new byte[len]; 
      for(int i = 0; i < len; i++)
	ba[i] = (byte)in_.read();
      return new String(ba); 
    }
}
