/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		CBus 0.01
 *
 */

package EDU.rutgers.cbus.presentation;

// JDK
import java.io.OutputStream;
import java.io.IOException;

// ACT
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * @since CBus 0.01
 */
public class CBusTypeWriter extends TypeWriter
{
  /**
   * @param Writer actual writer to perform writing
   *
   * @since CBus 0.01
   */
  public CBusTypeWriter(OutputStream out)
    {
      super(out);
    }

  /**
   * @since CBus 0.01
   */
  public void writeBoolean(boolean b) throws IOException
    {
      out_.write(b ? 1 : 0); 
    }

  /**
   * @since CBus 0.01
   */
  public void writeChar(char c) throws IOException
    {
      out_.write((int)c);
    }

  /**
   * @since CBus 0.01
   */
  public void writeOctet(byte b) throws IOException 
    {
      out_.write(b & 0xFF);
    }

  /**
   * @since CBus 0.01
   */
  public void writeShort(short s) throws IOException
    {
      out_.write(s & 0xFF);
      out_.write(s >> 8);
    }

  /**
   * @since CBus 0.01
   */
  public void writeUShort(short s) throws IOException 
    {
      // identical with writeShort
      out_.write(s & 0xFF);
      out_.write(s >> 8);
    }

  /**
   * @since CBus 0.01
   */
  public void writeLong(int l) throws IOException 
    {
      out_.write(l & 0xFF); l >>= 8;
      out_.write(l & 0xFF); l >>= 8;
      out_.write(l & 0xFF); l >>= 8;
      out_.write(l);
    }

  /**
   * @since CBus 0.01
   */
  public void writeULong(int l) throws IOException
    {
      // identical with writeLong
      out_.write(l & 0xFF); l >>= 8;
      out_.write(l & 0xFF); l >>= 8;
      out_.write(l & 0xFF); l >>= 8;
      out_.write(l);
    }

  /**
   * @since CBus 0.01
   */
  public void writeLongLong(long ll) throws IOException
    {
      writeLong((int)ll);
      writeLong((int)(ll >> 32));
    }

  /**
   * @since CBus 0.01
   */
  public void writeULongLong(long ull) throws IOException
    {
      writeLongLong((long)ull);
    }

  /**
   * @since CBus 0.01
   */
  public void writeFloat(float f) throws IOException
    {
      writeLong(Float.floatToIntBits(f));
    }

  /**
   * @since CBus 0.01
   */
  public void writeDouble(double d) throws IOException 
    {
      writeLongLong(Double.doubleToLongBits(d));
    }

  /**
   * @since CBus 0.01
   */
  public void writeString(String str) throws IOException
    {
      int len = str.length();
      byte[] ba = str.getBytes();
      writeLong(len);
      for (int i = 0 ; i < len ; i++)
	out_.write(ba[i]);
    }
}
