package EDU.rutgers.cbus.tools;

import java.awt.*;

import EDU.rutgers.cbus.clntsrv.Communicator;

import EDU.rutgers.cbus.presentation.MouseAdaptor;
import EDU.rutgers.cbus.presentation.MouseMotionAdaptor;
import EDU.rutgers.cbus.presentation.KeyAdaptor;
import EDU.rutgers.cbus.presentation.FocusAdaptor;
import EDU.rutgers.cbus.presentation.ComponentAdaptor;
import EDU.rutgers.cbus.presentation.ActionAdaptor;
import EDU.rutgers.cbus.presentation.TextAdaptor;
import EDU.rutgers.cbus.presentation.EventReplicator;

public class UnawareLinker
{
  protected Communicator comm_;

  public UnawareLinker(Communicator comm)
    {
      comm_ = comm;
    }

  public void link(Component c)
    {
      linkThis(c);
      if(c instanceof Container)
	{
	  Component[] cv = ((Container)c).getComponents();
	  for(int i = 0; i < cv.length; i++)
	    {
	      link(cv[i]);
	    }
	}
    }
  
  protected void linkThis(Component c)
    {
      System.out.println("Linking a component...");

      MouseAdaptor mouse = new MouseAdaptor(c);
      MouseMotionAdaptor mouseMotion = new MouseMotionAdaptor(c);
      KeyAdaptor key = new KeyAdaptor(c);
      FocusAdaptor focus = new FocusAdaptor(c);
      ComponentAdaptor component = new ComponentAdaptor(c);

      EventReplicator mouseR = new EventReplicator(comm_);
      EventReplicator mouseMotionR = new EventReplicator(comm_);
      EventReplicator keyR = new EventReplicator(comm_);
      EventReplicator focusR = new EventReplicator(comm_);
      EventReplicator componentR = new EventReplicator(comm_);
      
      c.addMouseListener(mouse);
      c.addMouseMotionListener(mouseMotion);
      c.addKeyListener(key);
      c.addFocusListener(focus);
      c.addComponentListener(component);
      
      mouseR.addCBusEventListener(mouse);
      mouse.addCBusEventListener(mouseR);
      
      mouseMotionR.addCBusEventListener(mouseMotion);
      mouseMotion.addCBusEventListener(mouseMotionR);
      
      keyR.addCBusEventListener(key);
      key.addCBusEventListener(keyR);
      
      focusR.addCBusEventListener(focus);
      focus.addCBusEventListener(focusR);
      
      componentR.addCBusEventListener(component);
      component.addCBusEventListener(componentR);

/*      c.enableEvents(AWTEvent.MOUSE_EVENT_MASK |
		     AWTEvent.MOUSE_MOTION_EVENT_MASK |
		     AWTEvent.KEY_EVENT_MASK |
		     AWTEvent.FOCUS_EVENT_MASK |
		     AWTEvent.COMPONENT_EVENT_MASK
		     );*/

      if(c instanceof Button)
	{
	  linkButton((Button)c);
	}
      else if(c instanceof TextComponent)
	{
	  linkTextComponent((TextComponent)c);
	}
    }

  protected void linkButton(Button c)
    {
      ActionAdaptor action = new ActionAdaptor(c);
      EventReplicator actionR = new EventReplicator(comm_);
      c.addActionListener(action);
      actionR.addCBusEventListener(action);
      action.addCBusEventListener(actionR);
    }

  protected void linkTextComponent(TextComponent c)
    {
      TextAdaptor text = new TextAdaptor(c);
      EventReplicator textR = new EventReplicator(comm_);
      c.addTextListener(text);
      textR.addCBusEventListener(text);
      text.addCBusEventListener(textR);

      if(c instanceof TextField)
	{
	  linkTextField((TextField)c);
	}
    }

  protected void linkTextField(TextField c)
    {
      ActionAdaptor action = new ActionAdaptor(c);
      EventReplicator actionR = new EventReplicator(comm_);
      c.addActionListener(action);
      actionR.addCBusEventListener(action);
      action.addCBusEventListener(actionR);
    }
}
