/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package ORG.omg.CORBA;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * All objects that need to be passed by copy rather than
 * by reference must implement this interface. It is the
 * responsibility of the subclass to provide correct
 * methods for the streamalization of objects. 
 * <p>
 * All streamable objects MUST have a constructor without 
 * arguments.
 *
 * @see EDU.rutgers.act.presentation.TypeReader
 * @see EDU.rutgers.act.presentation.TypeWriter
 *
 * @since ACT 0.1
 */
public interface Streamable
{
    /**
     * Read an external representation of this streamable object from
     * stream.
     *
     * @param TypeReader the specific type reader
     *
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void read(TypeReader in) 
		throws java.io.IOException;

    /**
     * Write an external representation of this streamable object to
     * stream.
     *
     * @param TypeWriter the specific type writer
     *
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void write(TypeWriter out)
		throws java.io.IOException;
}
