// for emacs only: -*- C++ -*-

// last modification 02/04/98

package examples.IIOP;

import java.io.*;

import ORG.omg.CORBA.Principal;
import EDU.rutgers.act.endpoint.Endpoint;
import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.presentation.*;
import EDU.rutgers.CORBA.presentation.*;
import EDU.rutgers.CORBA.GIOP.actions.*;
import EDU.rutgers.CORBA.GIOP.messages.*;
import EDU.rutgers.CORBA.IIOP.*;
import EDU.rutgers.CORBA.IOP.*;

class Client
{
  public static void main(String args[]) throws java.io.IOException
    {
      if(args.length != 2)
	{
	  System.out.println("usage: Client <hostname> <portnumber>");
	  throw new IOException("Invalid arguments");
	}

      FSMDataFactory giopClntFactory =
	(FSMDataFactory)new GIOPClientDataFactory();
      MessageFactory msgFactory = 
	(MessageFactory)new GIOPMessageFactory();

      Sender clientSender = new ClientSender(giopClntFactory,
					     msgFactory,
					     new CDRTransferSyntaxFactory());

      Endpoint dest = new Endpoint(args[0],
				   Integer.parseInt(args[1]));

      String[] str_ptr = new String[1];
      

      String operation;
      char command = readCommand(str_ptr);
      while(command != 'q')
	{
	  switch(command)
	    {
	    case 'e':
	      operation = "echo";
	      break;

	    case 'r':
	      operation = "revert";
	      break;
	  
	    case 'u':
	      operation = "toUpper";
	      break;
	  
	    case 'l':
	      operation = "toLower";
	      break;
	  
	    default:
	      System.out.println("Invalid operation - replacing it with eCHO");
	      operation = "echo";
	    }
      
	  Message msg = composeRequest("01".getBytes(),
				       operation,
				       str_ptr[0],
				       dest);
	  
	  System.out.println("Sending <"+msg+">");

	  clientSender.sendMessage(msg);
	  Receiver clientReceiver = new ClientReceiver(msg.getTransactionId());
	  GIOPReplyMessage reply = 
	    (GIOPReplyMessage)clientReceiver.receiveMessage();

	  //
	  // create the type reader for getting the content of the body
	  //
	  ByteArrayInputStream stream = 
	    new ByteArrayInputStream(reply.getReplyBody());
	  TypeReader in = new CDRTypeReader(new InputStreamReader(stream));

	  in.setIndex(reply.getBeforeBodySize());
	  in.setByteOrder(reply.byte_order_);


	  //
	  // read the data inside the body of the Reply message
	  //
	  String answer = "";
	  try {
	    answer = in.readString();
	    // ...
	    // read primitive types of data using TypeReader primitives
	  }
	  catch(java.io.IOException e)
	    {
	      System.out.println("Marshaling/unmarshaling error: " + e);
	      System.exit(1);
	    }

	  System.out.println("Answer is <" + answer + ">");

	  command = readCommand(str_ptr);
	}
    }

  static char readCommand(String[] str_ptr) throws IOException
    {
      byte[] line1 = new byte[256];
      byte[] line2 = new byte[256];

      System.out.print("string to be processed: ");
      int len = System.in.read(line1) - 1;
      str_ptr[0] = new String(line1, 0, len);
      System.out.print("command (eCHO, rEVERT, TOuPPER, TOlOWER): ");
      System.in.read(line2);
      return (char)line2[0];
    }

  static Message composeRequest(byte[] objectKey,
				String operation,
				String arg,
				Endpoint ep
				)
    {
      byte[] context_data = "".getBytes();
      ServiceContext[] service_context = new ServiceContext[0];

      for (int i = 0; i < service_context.length; i ++)
	{
	  service_context[i] = new ServiceContext(1, context_data);
        }

      // !!!! careful with request IDs!!!!
      int requestId = 1; // put the request ID here, unique ints will do; 
      boolean response_expected = true;
      Principal requesting_principal = new Principal("some string".getBytes());
      GIOPRequestMessage request =
	new GIOPRequestMessage(service_context, 
			       requestId, 
			       response_expected, 
			       objectKey, 
			       operation, 
			       requesting_principal);

      //
      // compose the body of the message
      //
      ByteArrayOutputStream bodyStream_ = new ByteArrayOutputStream();
      TypeWriter outWriter_ = 
	new CDRTypeWriter(new OutputStreamWriter(bodyStream_));
      outWriter_.setByteOrder(request.byte_order_); // inherited

      // aligned with the GIOP header
      outWriter_.setIndex(request.getBeforeBodySize()); 

      //
      // start writing the data inside
      //
      try
	{
          // write data inside the body using CDR primitives, e.g.:
          outWriter_.writeString(arg);
          // ... use other primitives as you wish...

	  outWriter_.flush();

	  // write to the body
	  byte[] bodyBytes;

	  bodyBytes = bodyStream_.toByteArray();
	  request.setRequestBody(bodyBytes);
        }
      catch (java.io.IOException e)
	{
	  System.err.println("Error composing Request Message");
	  System.exit(1);
        }

      request.setEndpoint(ep);

      return request;
    }
  
  
}
